/* XND_REXP.C   read the experimental parameters */

#include "xnd_def.h"
#include "xnd_rdh.h"
#define EPS_CHK 0.01

/* ------------------------------------------------------------------ */
void write_chk (P_(void))
{    OBS obs;
     obs.dtheta = CHK_VAL;
     obs.i_obs  = l_diag->temp;
     obs.weight = l_diag->w_mini;
     write_data (&obs);
     obs.i_obs  = l_diag->dtheta0;
     obs.weight = l_diag->dthetaf;
     write_data (&obs);
     }
/* ------------------------------------------------------------------ */
FLOAT read_chk(P_(void))
{    OBS obs;
     FLOAT t;
     read_data(&obs);
     tst_abort ((fabs((double)obs.dtheta-CHK_VAL)>EPS_CHK),"Verif data_fich");
     t = obs.i_obs;
     l_diag->w_mini = obs.weight;
     read_data(&obs);
     tst_abort ((fabs(obs.dtheta-CHK_VAL)>EPS_CHK),"Verif data_fich");
     l_diag->dtheta0 = obs.i_obs;
     l_diag->dthetaf = obs.weight;
     return t;
     }
/* ------------------------------------------------------------------ */
/* ------------------------------------------------------------------ */
static void pseudo_data (P_(void))
{    int    i, nb, num;
     OBS    obs;
	  float  step;
     
	 push_err_msg("pseudo_data(xnd_rexp.c)");
	 for (num=0; num < l_exper->n_diag; num++,l_diag++)
    {    l_diag->w_mini = 1;
         purge_cr();
			fscanf (coord_fich,"%f%f%f%f",&l_diag->temp,&l_diag->dtheta0,&step,&l_diag->dthetaf);
			fprintf (list_fich," PSEUDO_DIAG : T = %.3f, dtheta0=%.4f step=%.4f dthetaf=%.4f\n",
				 l_diag->temp,l_diag->dtheta0, step, l_diag->dthetaf);
			tst_abort ((step <= MINFLOAT),"step lower or egal to zero");
		  nb = 1 + floor ((l_diag->dthetaf-l_diag->dtheta0)/step);
        obs.dtheta = l_diag->dtheta0;
        obs.weight = obs.i_obs = 0;
        for (i=0;i<nb;i++,obs.dtheta += step) write_data (&obs);
        write_chk();
        l_diag->n_points = nb;
     lect->tot_points += l_diag->n_points;
     if (l_diag->n_points > lect->max_points)
				 lect->max_points = l_diag->n_points;
	  }
	pop_err_msg();
	}


/* ------------------------------------------------------------------ */
int read_exper (P_(void))
{   char   *txt_nature, *txt_techn, *txt_fich, *txt_powder;
    char   *pluriel;
    int    i, nature, n_tech, n_arg;

    push_err_msg("read_exper(xnd_rexp.c)");
    l_exper->remark = read_remark();
    n_arg = fscanf (coord_fich," %d%d%d%d%d%d%f",&nature,
	    &l_exper->techn,&l_exper->n_diag,&l_exper->dfich,
	    &l_exper->n_lambda, &l_exper->n_back,
	    &l_exper->temp_orig);
    purge_cr();
    n_arg += fscanf (coord_fich," %[^\n]", l_exper->nom);
    switch (nature % 8)
	{	case 0  :  l_exper->nature = EXPER_RX;
			   txt_nature = "RX"; break;
		case 1  :  l_exper->nature = EXPER_N;
			   txt_nature = "NEUTRONS"; break;
		case 2  :  l_exper->nature = EXPER_RXA;
			   txt_nature = "RX_ANOMAL"; break;
		case 3  :  l_exper->nature = EXPER_NM;
			   txt_nature = "NEUTRONS MAGNETIQUES"; break;
		default : txt_nature = "???";
		}
	if ( nature & 8 )
	{   txt_powder = " single crystal" ;
		 l_exper->nature |= EXPER_SINGLE_CRYSTAL;
		 }
	else txt_powder =	"powder";
	n_tech = 4;
	switch (l_exper->techn)
	{	case 0 :  txt_techn = "Reflexion Sym";
			  break;
		case 1 :  txt_techn = "Reflexion Assym";
			  n_tech++; break;
		case 2 :  txt_techn = "Reflexion Lineaire";
			  n_tech++; break;
		case 3 :  txt_techn = "Reflexion Banane";
			  n_tech++; break;
		case 4 :  txt_techn = "Transmission";
			  n_tech+=2; break;
		default : txt_techn = "???";
		}
	switch (l_exper->dfich)
	{	case 0 :  txt_fich = "SIMULATION"; break;
		case 1 : txt_fich = " 2th i"; break;
		case 2 : txt_fich = " th 2th i"; break;
		case 3 : txt_fich = " 2th i s"; break;
		case 4 : txt_fich = " th 2th i s"; break;
		case 10 :  txt_fich = "exRAW"; break;
		case 11 :  txt_fich = "DBW"; break;
		case 12 : case 13 :
		case 14 : txt_fich = "LLB"; break;
		case 20 : txt_fich = "ILL_RT" ; break;
		case 21 : txt_fich = "VDMI_RT" ; break;
		case 50 : txt_fich = "hkl Fo So"; break;
		case 51 : txt_fich = "hkl Io So"; break;
		default : txt_fich = "???";
		}
	if (l_exper->n_diag > 1) pluriel = "s"; else pluriel = "";
	nt_diag += l_exper->n_diag;

	fprintf(list_fich,"\n EXPER : %d (%s,%s), techn %d (%s)\n %d data set%s : file%s type %d (%s) \n\
 wavelength : %d, background : %d points\n T_orig in expansion %.3f\n '%s'\n",
		 nature, txt_nature, txt_powder,
		 l_exper->techn, txt_techn, l_exper->n_diag,
		 pluriel, pluriel, l_exper->dfich, txt_fich, l_exper->n_lambda,
		 l_exper->n_back, l_exper->temp_orig,	l_exper->nom);
	 if (bavard) printf("\n EXPER : '%s' : %d data set\n",
		 l_exper->nom, l_exper->n_diag);
	tst_abort( (n_arg!=8), "error reading experiment header", n_arg);      
	tst_abort ((nature<0 || (nature%8)>3),"unknown experimental nature");
    tst_abort ((l_exper->techn<0 || l_exper->techn>4),"unknown experimental setup");
	 tst_abort ((l_exper->n_diag==0),"Experiment without data set");
    
    l_diag = l_exper->diag = (DIAG *) tst_alloc(l_exper->n_diag,sizeof(DIAG),"patterns");
    switch  (l_exper->dfich)
    {   case 0  : pseudo_data ();   break;
	  case 20 :
	  case 21 : get_mult_data (); break;
	  case 50 : get_hkl_data (); break;
	  default : get_real_data ();
	  }

     fprintf(list_fich,"\n");
     fget_var (&l_exper->onde,LAMBDA*l_exper->n_lambda,LAMBDA,VAR_STD,T_LAMBDA,(void *) txt_onde);
     l_exper->prof_par = 0;
     VAR_init(&l_exper->onde);
     VAR_io_id
     for (i=0;i<l_exper->n_lambda;i++)
     {   if (var_param(i*LAMBDA+2)) l_exper->prof_par++;
	 if (var_param(i*LAMBDA+3)) l_exper->prof_par++;
	 }
     fget_var (&l_exper->back, l_exper->n_back, 0, VAR_BG, T_BACK, NULL);

	  fget_var (&l_exper->reglage, n_tech, 0, VAR_STD, T_REGLAGE,(void *) txt_reglage);
	  fget_var (&l_exper->instru, INSTRU, 0, VAR_STD,T_INSTRU,(void *) txt_instru);
	  fget_var (&l_exper->assym, max_assym*ASSYM, ASSYM, VAR_ORIEN,T_ASSYM, (void *) txt_assym);
	  VAR_init(&l_exper->instru);
	  VAR_io_id for (i=0;i<4;i++)	if (var_param(i)) l_exper->prof_par++;

 	switch (l_exper->nature)
	{	case EXPER_RX  :  return (LECT_NAT_RX);
		case EXPER_N   :  return (LECT_NAT_N);
		case EXPER_RXA :  return (LECT_NAT_RXA | LECT_NAT_RX);
		case EXPER_NM  :  return (LECT_NAT_NM | LECT_NAT_N);
		}
		pop_err_msg();
		return 0;
	}

/* ------------------------------------------------------------------ */

